def get_local_isos_and_storage(proxmox) -> tuple[list[str], list[str]]:
    iso_list = []
    proxmox_storage = []
    for node in proxmox.nodes.get():
        storage_list = proxmox.nodes(node["node"]).storage.get()
        for storage in storage_list:
            if storage["storage"] == "local":
                iso_content = (
                    proxmox.nodes(node["node"])
                    .storage(storage["storage"])
                    .content.get()
                )
                for content in iso_content:
                    if content["volid"] not in iso_list:
                        iso_list.append(content["volid"])
            else:
                if storage["storage"] not in proxmox_storage:
                    proxmox_storage.append(storage["storage"])
    return iso_list, proxmox_storage


def get_network_bridges(proxmox) -> list[str]:
    bridge_list = []
    for node in proxmox.nodes.get():
        network_list = proxmox.nodes(node["node"]).network.get()
        for network in network_list:
            if network["type"] == "bridge":
                if network["iface"] not in bridge_list:
                    bridge_list.append(network["iface"])
    return bridge_list



import os
import sys
import time
import keyboard
import subprocess


def check_packer_installed() -> None:
    try:
        packer_check = subprocess.run(
            ["packer", "--version"], capture_output=True, text=True
        )
        print(f"Packer is installed. Version: {packer_check.stdout.strip()}")
    except FileNotFoundError:
        print("Packer is not installed.")
        print(
            "Please install packer and make it executable to be able to use this package."
        )
        sys.exit(1)


def main() -> None:
    check_packer_installed()

    package_dir = os.path.dirname(os.path.abspath(__file__))
    app_path = os.path.join(package_dir, "../proxmoxtemplates/main.py")
    process = subprocess.Popen(["streamlit", "run", app_path])

    time.sleep(1)

    print()
    print("To quit the app, press 'strg + c'.")
    print()

    # Listen for 'strg+c' key press
    keyboard.add_hotkey("ctrl+c", lambda: process.terminate())

    # Wait for the Streamlit app process to complete
    process.wait()

    print("Streamlit app terminated.")


if __name__ == "__main__":
    main()



import streamlit as st
from proxmoxtemplates.frontend_utils.utils import setup_logger
from proxmoxtemplates.deployment_utils.deployment import deployment
from proxmoxtemplates.deployment_utils.create_obj import (
    PackerConfig,
    ProxmoxNode,
)


def template_form() -> None:
    with st.form(key="template_setup"):
        st.markdown("#### Setup of the template:")

        selected_nodes = st.multiselect(
            "One which node, should the template be placed?",
            st.session_state.my_proxmox.proxmox_nodes,
            placeholder="Choose nodes...",
        )
        ubuntu_os_version = st.selectbox(
            "Select the ubuntu os version",
            [
                "ubuntu-20-04-server",
                "ubuntu-22-04-server",
                "ubuntu-22-04-server-terraform",
                "ubuntu-24-04-server",
            ],
            placeholder="ubuntu-20-04, ...",
        )
        template_id = st.number_input(
            "Template VM ID (the id will incrementally increase):",
            min_value=1,
            max_value=999,
            placeholder="900",
        )
        template_name = st.text_input(
            "Template name", placeholder="template-ubuntu-22-04"
        )
        template_description = st.text_input(
            "Template description", placeholder="Ubuntu 22.04 template ...."
        )
        template_tags = st.text_input(
            "Template tags (Please input it, seperated with a comma)",
            placeholder="template, ubuntu, ...",
        )
        template_iso = st.selectbox(
            "Select a ISO (must be available on all selected nodes!)",
            st.session_state.my_proxmox.local_isos,
            index=None,
            placeholder="Select ISO...",
        )
        storage_pool = st.selectbox(
            "Select a storage pool.",
            st.session_state.my_proxmox.storage_pools,
            index=None,
            placeholder="Select a storage pool...",
        )

        col1, col2 = st.columns(2)

        with col1:
            disk_size = st.number_input(
                "choose disk size in G:",
                min_value=1,
                max_value=999,
                placeholder="32",
            )
            cores = st.number_input(
                "choose the core size:", min_value=1, max_value=999, placeholder="4"
            )
            keyboard_layout = st.text_input("Keyboard layout", placeholder="de")

        with col2:
            memory = st.number_input(
                "choose the memory size (MB):",
                min_value=1024,
                max_value=100000,
                placeholder="4092",
            )
            bridge = st.selectbox(
                "Select a Bridge (must be available on all nodes!)",
                st.session_state.my_proxmox.network_bridges,
                index=None,
                placeholder="Select bridge...",
            )
            timezone = st.text_input("Timezone", placeholder="Europe/Vienna")

        st.markdown(
            "##### Settings for ssh access (is needed for the packer connection):"
        )

        ssh_username = st.text_input("SSH Username", placeholder="TestSSH")
        path_to_ssh_key_file = st.text_input(
            "Path to your local ssh key file", placeholder="~/.ssh/homelab"
        )
        ssh_public_key = st.text_input(
            "Your public ssh key", placeholder="ssh-rsa xxxxxx"
        )

        packages = st.multiselect(
            label="Preinstall on template: ",
            options=["None", "docker & docker-compose"],
            default=["None"],
            placeholder="Select what you want...",
        )

        # Check if all required fields are filled
        required_fields = [
            selected_nodes,
            template_id,
            template_name,
            template_description,
            template_tags,
            disk_size,
            cores,
            keyboard_layout,
            memory,
            timezone,
            ssh_username,
            path_to_ssh_key_file,
            ssh_public_key,
        ]
        all_required_fields_filled = all(required_fields)

        template_submit = st.form_submit_button("Create Template(s)")

        if (
            template_submit
            and all_required_fields_filled
            and ubuntu_os_version
            and template_iso
            and storage_pool
            and bridge
        ):
            placeholder_for_logger = st.empty()
            logger, log_stream = setup_logger()
            with st.spinner("Packer Deployment in progress..."):
                deployment(
                    conf=PackerConfig(
                        ubuntu_version=ubuntu_os_version,
                        proxmox_api_url=f"https://{st.session_state.my_proxmox.proxmox_url}:8006/api2/json",
                        proxmox_api_token_id=f"{st.session_state.my_proxmox.proxmox_user}!{st.session_state.my_proxmox.proxmox_api_token_name}",
                        proxmox_api_token_secret=st.session_state.my_proxmox.proxmox_api_token_secret,
                        template_name=template_name,
                        template_description=template_description,
                        iso_file=template_iso,
                        iso_storage_pool="local",
                        disk_size=f"{disk_size}G",
                        storage_pool=storage_pool,
                        storage_pool_type="rbd",
                        cores=f"{cores}",
                        memory=f"{memory}",
                        network_bridge=bridge,
                        packer_bind_address=st.session_state.my_proxmox.local_ip,
                        ssh_username=ssh_username,
                        path_to_ssh_key_file=path_to_ssh_key_file,
                        ssh_public_key=ssh_public_key,
                        proxmox_nodes=[
                            ProxmoxNode(
                                node_name=selected_node, vm_id=int(template_id + i)
                            )
                            for i, selected_node in enumerate(selected_nodes)
                        ],
                        tags=template_tags,
                        keyboard_layout=keyboard_layout,
                        timezone=timezone,
                        tls_verification=(
                            "true"
                            if st.session_state.my_proxmox.verify_ssl
                            else "false"
                        ),
                        additional_packages=[] if "None" in packages else packages,
                    ),
                    logger=logger,
                    placeholder=placeholder_for_logger,
                    log_stream=log_stream,
                )
                st.success("Template was successfully created!")
        else:
            st.error("Please fill out all the fields to enable the submit button.")



from pydantic import BaseModel
from typing import List, Optional


class MyProxmox(BaseModel):
    proxmox_url: str
    proxmox_user: str
    proxmox_api_token_name: str
    proxmox_api_token_secret: str
    verify_ssl: bool
    local_ip: str
    proxmox_nodes: Optional[list[str]] = None
    local_isos: Optional[list[str]] = None
    storage_pools: Optional[list[str]] = None
    network_bridges: Optional[list[str]] = None


class ProxmoxNode(BaseModel):
    node_name: str
    vm_id: int


class PackerConfig(BaseModel):
    ubuntu_version: str
    proxmox_api_url: str
    proxmox_api_token_id: str
    proxmox_api_token_secret: str
    template_name: str
    template_description: str
    iso_file: str
    iso_storage_pool: str
    disk_size: str
    storage_pool: str
    storage_pool_type: str
    cores: str
    memory: str
    network_bridge: str
    packer_bind_address: str
    ssh_username: str
    path_to_ssh_key_file: str
    ssh_public_key: str
    proxmox_nodes: List[ProxmoxNode]
    tags: str
    keyboard_layout: str
    timezone: str
    tls_verification: str
    additional_packages: list[str]



import logging
from io import StringIO


def setup_logger() -> tuple[logging.Logger, StringIO]:
    logger = logging.getLogger("packer_deployment_logger")
    logger.setLevel(logging.DEBUG)
    log_stream = StringIO()
    log_handler = logging.StreamHandler(log_stream)
    log_handler.setLevel(logging.DEBUG)
    formatter = logging.Formatter("%(message)s")
    log_handler.setFormatter(formatter)
    logger.addHandler(log_handler)
    return logger, log_stream



import streamlit as st
from proxmoxtemplates.proxmox_form import proxmox_form
from proxmoxtemplates.template_form import template_form


def initialize_session_state() -> None:
    if "form_submitted" not in st.session_state:
        st.session_state.form_submitted = False
    if "form2_submitted" not in st.session_state:
        st.session_state.form2_submitted = False
    if "my_proxmox" not in st.session_state:
        st.session_state.my_proxmox = None


def main() -> None:
    initialize_session_state()
    st.title("Proxmox Template Wizard with Packer 🚀")
    if not st.session_state.form_submitted:
        proxmox_form()

    if st.session_state.form_submitted and not st.session_state.form2_submitted:
        st.success("Proxmox connection setup complete!")
        template_form()


if __name__ == "__main__":
    main()



import os
import subprocess
import time
from io import StringIO
from logging import Logger
from streamlit.delta_generator import DeltaGenerator
from .create_obj import PackerConfig
from .file_handling import (
    create_temp_folder,
    handle_node_file_structure,
    remove_temp_folder,
)
from .templating import (
    render_packer_file,
    render_user_data,
)


def deployment(
    conf: PackerConfig,
    logger: Logger,
    placeholder: DeltaGenerator,
    log_stream: StringIO,
) -> None:
    # get the root path of this file
    root_path = os.path.dirname(os.path.abspath(__file__))

    # change to the template folder
    template_folder_path = os.path.join(root_path, "deployment_template")

    # generate a temp folder
    folder_path = create_temp_folder(template_folder_path=template_folder_path)

    try:
        # the template file need to be generated per each node
        for node in conf.proxmox_nodes:
            # log
            logger.info("_________________________")
            logger.info(f"Starting {node.node_name} templating...")
            placeholder.text(log_stream.getvalue())
            tmp_path = handle_node_file_structure(
                root_template_path=template_folder_path,
                folder_path=folder_path,
                node=node,
            )

            # generate the templates
            render_packer_file(
                template_folder_path=template_folder_path,
                tmp_path=tmp_path,
                config=conf,
                node=node,
            )
            render_user_data(
                root_path=template_folder_path, tmp_path=tmp_path, config_obj=conf
            )

            # now cd to node dir
            os.chdir(tmp_path)

            # packer command
            logger.info(f"Starting {node.node_name} deployment...")
            placeholder.text(log_stream.getvalue())
            command = ["packer", "build", rf".\{node.node_name}.pkr.hcl"]
            process = subprocess.Popen(
                command,
                shell=True,
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE,
                text=True,
            )

            while process.stdout.readline() != "" and process.poll() is None:  # type: ignore
                logger.info(f"{process.stdout.readline()}")  # type: ignore
                placeholder.text(log_stream.getvalue())

            # change back to root temp folder
            os.chdir(template_folder_path)
    finally:
        time.sleep(1)
        remove_temp_folder(folder_path=folder_path)



import os
import shutil
from .create_obj import ProxmoxNode


def create_temp_folder(template_folder_path: str) -> str:
    folder_name = "tmp"
    folder_path = os.path.join(template_folder_path, folder_name)
    os.makedirs(folder_path)
    return folder_path


def handle_node_file_structure(
    root_template_path: str, folder_path: str, node: ProxmoxNode
) -> str:
    tmp_path = os.path.join(folder_path, node.node_name)
    os.makedirs(tmp_path)
    shutil.copytree(
        os.path.join(root_template_path, "files"), os.path.join(tmp_path, "files")
    )
    shutil.copytree(
        os.path.join(root_template_path, "http"), os.path.join(tmp_path, "http")
    )
    return tmp_path


def remove_temp_folder(folder_path: str) -> None:
    shutil.rmtree(folder_path)



import streamlit as st
from proxmoxer import ProxmoxAPI  # type: ignore
from proxmoxtemplates.deployment_utils.create_obj import MyProxmox
from proxmoxtemplates.proxmox_utils.utils import (
    get_local_isos_and_storage,
    get_network_bridges,
)


def handle_form_submit(my_proxmox: MyProxmox) -> MyProxmox:
    # Establish a connection to the Proxmox server
    proxmox = ProxmoxAPI(
        my_proxmox.proxmox_url,
        user=my_proxmox.proxmox_user,
        token_name=my_proxmox.proxmox_api_token_name,
        token_value=my_proxmox.proxmox_api_token_secret,
        verify_ssl=my_proxmox.verify_ssl,
        timeout=20,
    )
    my_proxmox.proxmox_nodes = [node["node"] for node in proxmox.nodes.get()]
    my_proxmox.local_isos, my_proxmox.storage_pools = get_local_isos_and_storage(
        proxmox
    )
    my_proxmox.network_bridges = get_network_bridges(proxmox)
    return my_proxmox


def proxmox_form() -> None:
    with st.form(key="proxmox_form"):
        st.write("Please setup the connection details for your Proxmox server:")

        proxmox_url = st.text_input("Url:", placeholder="10.10.0.10")

        proxmox_user = st.text_input("User:", placeholder="root@...")

        proxmox_api_token_name = st.text_input(
            "Token Name:", placeholder="YourTokenName"
        )

        proxmox_api_token_secret = st.text_input(
            "Token Secret:", placeholder="xxxxx-xxxx-..."
        )

        local_ip = st.text_input(
            "Your local IP to establish the connection:", placeholder="10.10.10.10"
        )

        verify_ssl = st.checkbox("SSL True", value=False)

        submit_button = st.form_submit_button("Set Proxmox Connection")

        # Conditional logic for enabling the form submission
        if submit_button and all(
            [
                proxmox_url,
                proxmox_user,
                proxmox_api_token_name,
                proxmox_api_token_secret,
                verify_ssl,
                local_ip,
            ]
        ):
            with st.spinner("Establishing the connection to proxmox..."):
                my_proxmox = handle_form_submit(
                    my_proxmox=MyProxmox(
                        proxmox_url=proxmox_url,
                        proxmox_user=proxmox_user,
                        proxmox_api_token_name=proxmox_api_token_name,
                        proxmox_api_token_secret=proxmox_api_token_secret,
                        verify_ssl=verify_ssl,
                        local_ip=local_ip,
                    )
                )
                st.session_state.my_proxmox = my_proxmox
                st.session_state.form_submitted = True
                st.rerun()
        else:
            st.error("Please fill out all the fields to enable the submit button.")



import os
from setuptools import setup, find_packages

version_string = os.environ.get("PACKAGE_VERSION", "0.0.0.dev0")

setup(
    name="proxmoxtemplates",
    version=str(version_string),
    packages=find_packages(),
    install_requires=[
        "streamlit==1.34.0",
        "proxmoxer==2.0.1",
        "pydantic==2.7.1",
        "jinja2==3.1.4",
        "keyboard==0.13.5",
    ],
    entry_points={
        "console_scripts": [
            "proxmoxtemplates_app=bin.run_app:main",
        ],
    },
    include_package_data=True,
)



import os
from jinja2 import Environment, FileSystemLoader
from .create_obj import PackerConfig, ProxmoxNode


def render_packer_file(
    template_folder_path: str, tmp_path: str, config: PackerConfig, node: ProxmoxNode
) -> None:
    # render
    template = Environment(
        loader=FileSystemLoader(os.path.join(template_folder_path, "jinja_templates"))
    ).get_template("base_template.pkr.hcl.j2")
    rendered_content = template.render(config=config, node=node)

    # save
    with open(
        (os.path.join(tmp_path, f"{node.node_name}.pkr.hcl")), "w"
    ) as packer_file:
        packer_file.write(rendered_content)


def render_user_data(root_path: str, tmp_path: str, config_obj: PackerConfig) -> None:
    # render
    template = Environment(
        loader=FileSystemLoader(os.path.join(root_path, "jinja_templates"))
    ).get_template("user-data.j2")
    rendered_content = template.render(config=config_obj)

    # save
    with open((os.path.join(tmp_path, "http", "user-data")), "w") as user_file:
        user_file.write(rendered_content)
